<?php
/**** ad_classes.php:  This file contains extra classes that can be used for in game objects and things. ****/

/***
	PHPAdventure!   Copyright (C) 2003 Steven Kollmansberger

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
***/

/*************
	Room: An object where the charactor can be.  A room is similiar to a container, but is designed for charectors to be in and travel
	 from one room to another.

	 New/Updated Properties:
	 	$cango: lists acceptable destination rooms for travel.  Use addGo to add an entry and output a link at the same time.
		$firstseen:  displayed when the first time a player enters a room
		$linkPref: prefaces an addgo link (to help the user see that the link is to go, not to look)

	New/Updated Overloadable Methods:
		enterRoom(&$origoid):  This function is called when a room is entered.  Any output should be returned as a variable (which will then
			be displayed) so that the system knows to pause.  Non-visible changes do not need to return anything.  You can set
			origoid to NULL to about travel, but other changes will be ignored.
		leaveRoom(&$destoid):  This function is called when a room is left.  Any output should be returned as a variable (which will then
			be displayed) so that the system knows to pause.  Non-visible changes do not need to return anything.
			Note that you can alter the destination or even cancel travel by setting it to NULL.  It's generally
			nice to display a message if you do. :)
		firstSeen():  Called the first time a player enters a room.  Returns text to be displayed.

	New/Updated Useful Methods:
		addGo($destoid, $title): use this function to add a room destination and create a link at the same time.  Note that $dest is NOT the room
			object itself but merely the name of the room.  This is done so that rooms can connect to each other without
			the room needing to exist first (catch-22).  When this function is called, it returns a link with the specified title.
			Calling this function multiple times for one room is no problem.

	New/Updated Verbs:
		look: as before, but does not display any verbs.
		go: moves to another room, if appropriate.




***********/

class Room extends Object {
	var $cango;
	var $firstseen;
	var $hasseen;
	var $linkPref;

	function firstSeen() { return $this->firstseen; }
	function aName() { return $this->Name(); } // because rooms aren't a
	/* function theName() { return $this->Name(); } */
	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc . "<br>" . $this->listContents();
		return $a;
	}
	function addGo($destoid, $title) {

		if (array_key_exists($destoid, $this->cango) == FALSE) {
			$this->cango[$destoid] = $destoid;
		}

		$a = "<a href='" . BaseURL() . "_item=";
		$a = $a . $destoid;
		$a = $a . "&_verb=go";
		$a = $a . "'";

		$a = $a . ">";
		$a = $a . $this->linkPref . $title;
		$a = $a . "</a>";

		return $a;

	}
	function listContents() { // not totally identical to openable listcontents

		global $_allobjs;
		$tmp = array();
		foreach ($this->contains as $k => $i) {
			if ($_allobjs[$i]->isListed()) {
				$tmp[$k] = $i;
			}
		}

		if (count($tmp) == 0) {
			return "";
		}

		$a = "You see ";
		$b = "";
		foreach ($tmp as $i) {
			if ($c > "") $b = ", " . $b;
			$c = $c . $b;
			$f = $_allobjs[$i];
			if ($f->doVerbVerify("look"))
				$b = $f->objRef("look", $f->aName());
			else
				$b = $f->aName();


		}
		$a = $a . $c;
		if ($c > "") $a = $a . " and " ;
		$a = $a . $b;
		if ($a > "") $a = $a . " here.  ";
		// recurse, if needed
		foreach ($tmp as $i) {
			$f = $_allobjs[$i];
			$tmp2 = array();
			foreach ($f->contains as $k => $i) {
				if ($_allobjs[$i]->isListed()) {
					$tmp2[$k] = $i;
				}
			}
			if (count($tmp2) > 0) {
				$a = $a . $f->listContents();
			}
			//if (count($f->contains) > 0) {
			//	$a = $a . $f->listContents();
			//}
		}
		return $a;
	}
	function Room() {
		Object::Object();

		array_push($this->doVerbs, "go");
		$this->cango = array();
		$this->hasseen = FALSE;
		$this->linkPref = "&raquo;&nbsp;";
	}
	function doVerbVerify($verb) {
		global $_allobjs;
		if ($verb == "go") {
			$f = $_allobjs[$_allobjs['_Me']->location];

			if (array_key_exists($this->oid, $f->cango)) {
				return TRUE;
			} else {
				return FALSE;
			}
		}

		return parent::doVerbVerify($verb);
	}

	function enterRoom(&$origoid) {
		// overload this function to perform actions when the player enters the room
		// return any output and the system will pause to display it

	}

	function leaveRoom(&$destoid) {
		// overload this function to perform actions when the player leaves the room
		// return any output and the system will pause to display it

	}

	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "look":
			// We override look because we don't want to list "go" or other verbs
			// and we want firstseen support

			if ($this->hasseen == FALSE) {
				$a = $this->Firstseen();
				if (!is_null($a)) {
					print "<div class='descpane'>" . dynamicString($a);
					print "</div>";
				}
				$this->hasseen = TRUE;
			}
			//print "<div class='descpane'>" . $this->Desc();
			//print "</div>";
			parent::doVerbHandle($verb);

			//print $this->showVerbs($verb);
			break;
		case "go":
			// go will be executed in the target class
			// must find _Me and see if the source class can get here
			$f =& $_allobjs['_Me'];

			$f2 =& $_allobjs[$f->location];
			$f3 = $this->oid;
			$pause = FALSE;
			// First, check for leaveroom results
			$a = $f2->leaveRoom($f3);

			if (!is_null($a)) {
				print "<div class='descpane'>" .dynamicString($a) . "</div>";
				$pause = TRUE;
			}

			if (!is_null($f3)) {
				$f4 = $f2->oid; // make a copy
				$a = $_allobjs[$f3]->enterRoom($f4); // otherwise it will be destroyed!
				if (!is_null($f4)) {

					$f->moveInto($_allobjs[$f3]);
				}

				if (!is_null($a)) {
					print "<div class='descpane'>" .dynamicString($a) . "</div>";
					$pause = TRUE;
				}
			}



			// we need to update the thingy, so reload the page
			if (strlen($_allobjs['_interlude']) > 0) {
				print dynamicString($_allobjs['_interlude']);
				$_allobjs['_interlude'] = "";
				$pause = TRUE;
			}
			if ($pause == FALSE && $_allobjs['_pause'] != TRUE) {
				print "<head><meta http-equiv='refresh' content='0; URL=" . $_SERVER['PHP_SELF'] . "'></head>";
				print "<div class='continue'><a href='" . $_SERVER['PHP_SELF'] . "'>Redirect</a></div>";
			}

			//print "<div class='descpane'>" . $this->Desc();
			//print "</div>";


			break;
		default:
			parent::doVerbHandle($verb);
		}
	}
	
	function verbIng($verb) {

		switch ($verb) {
		case "go":
			return "going to " . $this->theName();
		default:
			return parent::verbIng($verb);
		}
	}

}

/************
	Darkroom:  A room that requires light to operate in.  The only operation (besides inventory and system verbs)
	that will be allowed is to go "out" which will send the player back wherever they came from
	if they can go there.  If not, they are stuck.
	
	A Darkroom will be illuminated (act a normal room) if any object is present with the property
	emitsLight set to TRUE.  You can set this property of the darkroom to true and it will act as
	a normal room.
	NOTICE:  Light search is applied to the room and to _Me non-recursively.  Light sources inside
	containers (even transparent or open ones) will not count.  This should be changed sometime,
	but I'm too lazy to figure out all the details involved right now.
	NOTICE:  In general, this class is pretty rough.

	New/Updated Properties:
	$emitsLight:  set to true to make this room like a normal room
	$darkname:  The name given to this room when it is dark.  NULL means it retains the original name.
	$darkdesc:  The description given to this when it is dark.
	$outoid: Defines where "out" is.  Automatically set in enterRoom
	$outtitle: Indicates the title for the out link.
	
	Useful Methods:
	lightIsPresent(): Returns TRUE if the room is illuminated.

**********/

class Darkroom extends Room {
	var $emitsLight;
	var $darkname;
	var $darkdesc;
	var $outoid;
	var $outtitle;
	function Darkroom() {
		$this->emitsLight = FALSE;
		$this->darkname = "In the dark...";
		$this->darkdesc = "You are surrounded by pitch black darkness, and can't see a thing! ";
		$this->outtitle = "Go back.";
		$this->outoid = "";
		Room::Room();
	}
	function theName() {
		if ($this->lightIsPresent() || is_null($this->darkname)) return parent::theName();

		return "the dark";
	}
	function emitsLight() {
		return $this->emitsLight;
	}
	function lightIsPresent() {
		// check for the presence of a light source
		global $_allobjs;

		if ($this->emitsLight()) return TRUE;
		foreach ($this->contains as $i) {
			if ($_allobjs[$i]->emitsLight()) return TRUE;
		}
		foreach ($_allobjs['_Me']->contains as $i) {
			if ($_allobjs[$i]->emitsLight()) return TRUE;
		}
		return FALSE;

	}
	function Name() {

		if ($this->lightIsPresent() || is_null($this->darkname)) return parent::Name();

		return $this->darkname;
	}
	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "look":
			if ($this->lightIsPresent()) return parent::doVerbHandle($verb);
			$a = $this->darkdesc . " ";
			if (array_key_exists($this->outoid, $this->cango)) {
				$a = $a . "<a href='" . BaseURL() . "_item=";
				$a = $a . $this->outoid;
				$a = $a . "&_verb=go";
				$a = $a . "'>";
				$a = $a . $this->outtitle;
				$a = $a . "</a>";
			} else {
				$a = $a . "You are trapped!";
			}
			print "<div class='descpane'>" . dynamicString($a) . "</div>";
			break;
		default:
			return parent::doVerbHandle($verb);
		}
	}

	function isReachable($dest) {
		if ($this->lightIsPresent()) return parent::isReachable($dest);
		return FALSE;
	}
	function enterRoom(&$origoid) {
		$this->outoid = $origoid;
		return parent::enterRoom($orig);
	}



}

/*************
	Carryable:  An object which can be picked up and dropped, and carried around in the player's inventory
		Note that almost all types of game objects are derived from Carryable.  Obviously, you will not
		want the player to be able to pick up all these objects, so you can turn off the Carryable functionality
		in a given object.

	New/Updated Properties:
		$isFixed: set this property to true to disable the Carryable features in a given object.
		$isListed: as before, except that it defaults to NULL.  When NULL, the object is listed if not fixed, and not listed if fixed.
			A particular true/false value here overrides this.
		$isKey:  should this item be listed as a possible key (usually leave as TRUE).

	New/Updated Verbs:
		take: moves the item into the inventory
		drop: moves the item to the player's current location

	Can be Direct Object for:
		putin



**********/



class Carryable extends Object {
	// this kind will allow to be taken and dropped

	var $isFixed; // set to true to disable carryable features
	var $isKey;

	function isListed() {

		if (is_null($this->isListed)) return !$this->isFixed;
		return $this->isListed;
	}
	function Carryable() {


		Object::Object();

		$this->isFixed = FALSE;
		$this->isListed = NULL; // now null
		$this->isKey = TRUE;
		array_push($this->doVerbs, "take", "drop");
		array_push($this->ioVerbs, "lockwith", "unlockwith");
	}

	function doVerbVerify($verb) {
		global $_allobjs;
		if ($verb == "take") {
			return (!$this->isIn($_allobjs['_Me']) && !$this->isFixed);
		}
		if ($verb == "drop") {
			return ($this->isIn($_allobjs['_Me']) && !$this->isFixed);
		}
		if ($verb == "putin") {
			// this occurs when this is the dobj to a putin
			return !$this->isFixed;
		}

		return parent::doVerbVerify($verb);
	}

	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "take":
			$me =& $_allobjs['_Me'];
			$this->moveInto($me);
			print "<div class='descpane'>Taken.</div>";

			break;
		case "drop":
			$me =& $_allobjs['_Me'];
			$ml =& $_allobjs[$me->location];
			$this->moveInto($ml);
			print "<div class='descpane'>Dropped. </div>";

			break;
		case "putin":

			// the dobj will be the first to receive the command
			print $this->showIoObjs($verb);
			break;

		default:
			parent::doVerbHandle($verb);
		}
	}

	function verbIng($verb) {
		switch ($verb) {
		case "take":
			return "taking " . $this->theName();
		case "drop":
			return "dropping " . $this->theName();
		case "putin":
			return "putting " . $this->theName() . " in/on...";

		default:
			return parent::verbIng($verb);
		}
	}

	function verbDo($verb) {
		switch ($verb) {
		case "putin":
			return "Put " . $this->theName() . " in/on...";

		default:
			return parent::verbDo($verb);
		}
	}

	function verbIoIng($verb, $dobj) {
		switch ($verb) {
		case "lockwith":
			return "Locking " . $dobj->theName() . " with " . $this->theName();
		case "unlockwith":
			return "Unlocking " . $dobj->theName() . " with " . $this->theName();
		default:
			return parent::verbIoIng($verb, $dobj);
		}
	}
	
	function verbIo($verb, $dobj) {
		global $_allobjs;
		switch ($verb) {
		case "lockwith":
			return "Lock " . $dobj->theName() . " with " . $this->theName();
		case "unlockwith":
			return "Unlock " . $dobj->theName() . " with " . $this->theName();
		default:
			return parent::verbIo($verb, $dobj);
		}
	}

	function ioVerbVerify($verb, $dobj) {
		switch ($verb) {
		case "unlockwith":
		case "lockwith":
			return !$this->isFixed && $this->isKey;
		default:
			return parent::ioVerbVerify($verb, $dobj);
		}
	}
	
	function ioVerbHandle($verb, &$dobj) {
		switch ($verb) {
		case "unlockwith":
			if (array_key_exists($this->oid, $dobj->keyObjs)) {
				$dobj->isLocked = FALSE;
				print "<div class='descpane'>Unlocked.</div>";
			} else {
				print "<div class='descpane'>" . ucfirst(dynamicString($this->theName())) . " doesn't fit the lock.</div>";
			}
			break;
		case "lockwith":
			if (array_key_exists($this->oid, $dobj->keyObjs)) {
				$dobj->isLocked = TRUE;
				print "<div class='descpane'>Locked.</div>";
			} else {
				print "<div class='descpane'>" . ucfirst(dynamicString($this->theName())) . " doesn't fit the lock.</div>";
			}
			break;
		default:
			return parent::ioVerbHandle($verb, $dobj);
		}

	}

}

/***********
	Container:  An object which can contain other objects

	New/Updated Properties:
		isQuiet:  If set to true, items inside the container will not be listed except when looking
			directly at the container.

	Useful overloadable methods:
		contentsChanged($entered, &$obj):  Called when an object is added or removed from the contents
			entered will be TRUE if the object is added or FALSE otherwise.  obj will be the object in question.

	New/Updated Verbs:
		putin: puts another Carryable (that is not fixed) into this container
			Objects can be removed from a container by putting them in another container
			or executing take on them

	

	
******/

class Container extends Carryable {
	var $isQuiet;

	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc;
		$a = $a . "<br>" . $this->listContents(TRUE);
		return $a;
	}
	function listContents($lookdir = FALSE) {
		if ($lookdir == FALSE && $this->isQuiet) return "";
		global $_allobjs;
		$tmp = array();
		foreach ($this->contains as $k => $i) {
			if ($_allobjs[$i]->isListed()) {
				$tmp[$k] = $i;
			}
		}

		if (count($tmp) == 0) {
			return ucfirst($this->theName()) . " is empty. ";
		}
		$a = "Inside " . $this->theName() . " you see ";
		$b = "";
		foreach ($tmp as $i) {
			if ($c > "") $b = ", " . $b;
			$c = $c . $b;
			$f = $_allobjs[$i];

			$b = $f->objRef("look", $f->aName());


		}
		$a = $a . $c;
		if ($c > "") $a = $a . " and " ;
		$a = $a . $b;
		if ($a > "") $a = $a . ".  ";
		// recurse, if needed
		foreach ($tmp as $i) {
			$f = $_allobjs[$i];
			$tmp2 = array();
			foreach ($f->contains as $k => $i) {
				if ($_allobjs[$i]->isListed()) {
					$tmp2[$k] = $i;
				}
			}
			if (count($tmp2) > 0) {
				$a = $a . $f->listContents();
			}
		}
		return $a ;
	}

	function Container() {


		Carryable::Carryable();
		array_push($this->ioVerbs, "putin");
		$this->isQuiet = FALSE;

	}

	function ioVerbVerify($verb, $dobj) {
		if ($verb == "putin") {
			// we could check dobj to allow only certain types of objects
			// to be put in the container

			if (array_key_exists($dobj->oid, $this->contains)) return FALSE; // only at top level, lower is ok!

			return TRUE;
		}
		return parent::ioVerbVerify($verb, $dobj);
	}

	function ioVerbHandle($verb, &$dobj) {
		global $_allobjs;
		switch ($verb) {
		case "putin":
			$dobj->moveInto($this);
			print "<div class='descpane'>Done.</div>";
			break;
		default:
			parent::ioVerbHandle($verb, $dobj);
		}
	}


	function verbIo($verb, $dobj) {
		global $_allobjs;
		switch ($verb) {
		case "putin":
			return "Put " . $dobj->theName() . " in " . $this->theName();
		default:
			return parent::verbIo($verb, $dobj);
		}
	}

	function verbIoIng($verb, $dobj) {
		switch ($verb) {
		case "putin":
			return "Putting " . $dobj->theName() . " in " . $this->theName();
		default:
			return parent::verbIoIng($verb, $dobj);
		}
	}


}

/**************

	Surface:  A container that things go "on" instead of "in".  Otherwise exactly identical to a container

**************/

class Surface extends Container {
	function listContents($lookdir = FALSE) {
		if ($lookdir == FALSE && $this->isQuiet) return "";
		global $_allobjs;
		$tmp = array();
		foreach ($this->contains as $k => $i) {
			if ($_allobjs[$i]->isListed()) {
				$tmp[$k] = $i;
			}
		}

		if (count($tmp) == 0) {
			return "There's nothing on " . $this->theName() . ". ";
		}
		$a = "On " . $this->theName() . " you see ";
		$b = "";
		foreach ($tmp as $i) {
			if ($c > "") $b = ", " . $b;
			$c = $c . $b;
			$f = $_allobjs[$i];

			$b = $f->objRef("look", $f->aName());


		}
		$a = $a . $c;
		if ($c > "") $a = $a . " and " ;
		$a = $a . $b;
		if ($a > "") $a = $a . ".  ";
		// recurse, if needed
		foreach ($tmp as $i) {
			$f = $_allobjs[$i];
			$tmp2 = array();
			foreach ($f->contains as $k => $i) {
				if ($_allobjs[$i]->isListed()) {
					$tmp2[$k] = $i;
				}
			}
			if (count($tmp2) > 0) {
				$a = $a . $f->listContents();
			}
		}
		return $a ;
	}
	function verbIo($verb, $dobj) {
		global $_allobjs;
		switch ($verb) {
		case "putin":
			return "Put " . $dobj->theName() . " on " . $this->theName();
		default:
			return parent::verbIo($verb, $dobj);
		}
	}
	function verbIoIng($verb, $dobj) {
		switch ($verb) {
		case "putin":
			return "Putting " . $dobj->theName() . " on " . $this->theName();
		default:
			return parent::verbIoIng($verb, $dobj);
		}
	}
}

/**************

	Openable:  A container which can be open and closed.
	New/Updated Properties:
		$isopen:  Indicates if the container is currently open. Defaults to false.
		$isTransparent:  If TRUE, items can be seen within the container even when the container is closed.

	New/Updated Methods:
		isReachable($dest):  Does not allow access through a closed container.

	New/Updated Verbs:
		open: Opens the container.
		close: Closes the container.

**********/

class Openable extends Container {
	var $isTransparent;
	var $isopen;
	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc . "  " . ucfirst($this->theName()) . " is " . ($this->isopen ? "open. " : "closed. ");
		$a = $a . "<br>" . $this->listContents();
		return $a;
	}
	function aName() {
		if ($this->isopen) return "an open " . $this->Name(); else return "a closed " . $this->Name();
	}
	function listContents() {
		if ($this->isopen == FALSE && $this->isTransparent == FALSE) return "";
		return parent::listContents();
	}
	function isReachable($dest) { // overloaded!  closed containers don't have reacable things inside

		if ($this->isopen == FALSE)
			return FALSE;
		else
			return parent::isReachable($dest);
	}
	function Openable() {


		Container::Container();

		array_push($this->doVerbs, "open", "close");
		$this->isopen = FALSE;
		$this->isTransparent = FALSE;
	}

	function doVerbVerify($verb) {

		if ($verb == "open") {
			return !$this->isopen;
		}
		if ($verb == "close") {
			return $this->isopen;
		}
		return parent::doVerbVerify($verb);
	}

	function ioVerbVerify($verb, $dobj) {
		if ($verb == "putin") {
			if ($this->isopen == FALSE) return FALSE; // otherwise return the parent's opinion
		}
		return parent::ioVerbVerify($verb, $dobj);
	}

	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "open":
			$this->isopen = TRUE;
			print "<div class='descpane'>Opened. " . $this->listContents() . "</div>";

			break;
		case "close":
			$this->isopen = FALSE;
			print "<div class='descpane'>Closed. </div>";

			break;

		default:
			parent::doVerbHandle($verb);
		}
	}


	function verbIng($verb) {
		switch ($verb) {
		case "close":
			return "closing " . $this->theName();
		default:
			return parent::verbIng($verb);
		}
	}


}
/**************

	Switchable:  A carryable which can be switched on and off
	New/Updated Properties:
		$ison: Indicates if the item is switched on.  Defaults to TRUE.
		$switchlight:  Indicates that the item is emits light if it is switched on.  Defaults to FALSE.

	New/Updated Methods:


	New/Updated Verbs:
		turnon: Turns on the switch
		turnoff: Turns off the switch

**********/

class Switchable extends Carryable {
	var $ison;
	var $switchlight;
	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc . " " . ucfirst($this->theName()) . " is currently switched " . ($this->ison ? "on. " : "off. ");
		return $a;
	}

	function Switchable() {
		Carryable::Carryable();
		$this->ison = TRUE;
		$this->switchlight = FALSE;
		array_push($this->doVerbs, "turnon", "turnoff");

	}
	function emitsLight() {
		if ($this->switchlight == FALSE) return parent::emitsLight();
		return $this->ison;
	}
	function doVerbVerify($verb) {

		if ($verb == "turnon") {
			return !$this->ison;
		}
		if ($verb == "turnoff") {
			return $this->ison;
		}
		return parent::doVerbVerify($verb);
	}

	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "turnon":
			$this->ison = TRUE;
			print "<div class='descpane'>Turned on.</div>";

			break;
		case "turnoff":
			$this->ison = FALSE;
			print "<div class='descpane'>Turned off. </div>";

			break;

		default:
			parent::doVerbHandle($verb);
		}
	}


	function verbIng($verb) {
		switch ($verb) {
		case "turnon":
			return "turning on " . $this->theName();
		case "turnoff":
			return "turning off " . $this->theName();
		default:
			return parent::verbIng($verb);
		}
	}
	function verbDo($verb) {
		switch ($verb) {
		case "turnon":
			return "Turn on " . $this->theName();
		case "turnoff":
			return "Turn off " . $this->theName();
		default:
			return parent::verbDo($verb);
		}
	}


}

/**************
	NoEntranceRoom:  not a room at all, but rather an easy way to provide links with the "go" verb
		that never allow people to follow them.  if you want a room that sometimes allows enterance,
		this class is not for you.

	New/Updated Properties:
		$rejectMsg:  contains the message to display if the player tries to go there.


	Overloadable Methods:
		rejectMsg():  overload this if you want dynamic messages.  return the message to be displayed.

************/


class NoEntranceRoom extends Object {

	var $rejectMsg;

	function rejectMsg() { return $this->rejectMsg; }
	function NoEntranceRoom() {
		Object::Object();

		array_push($this->doVerbs, "go");
		$this->cango = array();
		$this->hasseen = FALSE;
		$this->rejectMsg = "You can't go there.";
	}
	function doVerbVerify($verb) {
		global $_allobjs;
		if ($verb == "go") {
			$f = $_allobjs[$_allobjs['_Me']->location];

			if (array_key_exists($this->oid, $f->cango)) {
				return TRUE;
			} else {
				return FALSE;
			}
		}

		return parent::doVerbVerify($verb);
	}

	function doVerbHandle($verb) {
		if ($verb == "go") {
			print "<div class='descpane'>" . dynamicString($this->rejectMsg()) . "</div>";

		} else return parent::doVerbHandle($verb);
	}

	function verbIng($verb) {

		switch ($verb) {
		case "go":
			return "going to " . $this->Name();
		default:
			return parent::verbIng($verb);
		}
	}
}

/**************
	Lockable:  An openable which can be locked and unlocked, optionally requiring a key for one
		or both of these operations.

	New/Updated Properties:
		$isLocked:  Set to TRUE if the object is currently locked.
		$keyObjs:  Contains a list of oids of the possible keys.  Use addKey() to set.
		$keylessLock:  Set to TRUE if the object does not require a key to be locked.
		$keylessUnlock:  Set to TRUE if the object does not require a key to be unlocked.
		
		Note:  If the lock can be operated in a certain way (lock or unlock) without a key,
			use of a key will not be allowed for that way.



	New/Updated Methods:
		addKey($obj):  Adds the specified object to the keyObjs list


	New Verbs:
		lock and unlock (for keyless operation)
		
	can be direct object for
		lockwith and unlockwith


**************/


class Lockable extends Openable {
	var $isLocked;
	var $keyObjs;
	var $keylessLock;
	var $keylessUnlock;

	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc . "  " . ucfirst($this->theName()) . " is " . ($this->isopen ? "open. " : ($this->isLocked ? "closed and locked. " : "closed. "));
		$a = $a . "<br>" . $this->listContents();
		return $a;
	}

	function addKey($obj) {
		$this->keyObjs[$obj->oid] = $obj->oid;
	}

	function Lockable() {
		Openable::Openable();
		$this->isLocked = FALSE;
		$this->keylessLock = TRUE;
		$this->keylessUnlock = TRUE;
		$this->keyObjs = array();
		array_push($this->doVerbs, "unlock", "lock");

	}

	function doVerbVerify($verb) {
		switch ($verb) {
		case "unlock":
			return ($this->isLocked && $this->keylessUnlock && !$this->isopen);
		case "lock":
			return (!$this->isLocked && $this->keylessLock && !$this->isopen);

		case "unlockwith":
			return ($this->isLocked && !$this->keylessUnlock && !$this->isopen);
		case "lockwith":
			return (!$this->isLocked && !$this->keylessLock && !$this->isopen);

		default:
			return parent::doVerbVerify($verb);
		}
	}
	


	function doVerbHandle($verb) {
		global $_allobjs;
		switch ($verb) {
		case "open":
			if ($this->isLocked) {
				print "<div class='descpane'>It's locked.</div>";
			} else {
				parent::doVerbHandle($verb);
			}

			break;
			
		case "lock":
			$this->isLocked = TRUE;
			print "<div class='descpane'>Locked.</div>";
			break;

		case "unlock":
			$this->isLocked = FALSE;
			print "<div class='descpane'>Unlocked.</div>";
			break;

		case "lockwith":
		case "unlockwith":
			print $this->showIoObjs($verb);
			break;

		default:
			parent::doVerbHandle($verb);
		}
	}

	function verbIng($verb) {
		switch ($verb) {
		case "unlock":
			return "Unlocking " . $this->theName();
		case "lock":
			return "Locking " . $this->theName();

		case "unlockwith":
			return "Unlocking " . $this->theName() . " with...";
		case "lockwith":
			return "Locking " . $this->theName() . " with...";

		default:
			return parent::verbIng($verb);
		}

	}

	function verbDo($verb) {
		switch ($verb) {
		case "unlock":
			return "Unlock " . $this->theName();
		case "lock":
			return "Lock " . $this->theName();
		case "unlockwith":
			return "Unlock " . $this->theName() . " with...";
		case "lockwith":
			return "Lock " . $this->theName() . " with...";
		default:
			return parent::verbDo($verb);
		}
	}

}

/********
	Door: A method for connecting two (or more) rooms that can impede progress.

	All Lockable properties apply

	New/Updated properties:
	$isLockable:  if TRUE, the user can lock and unlock the door
	$autoOpen:  if TRUE, the user will automatically unlock and open the door, if possible


	New/Updates methods:
	connectRooms(&$room1, &$room2):  connects two rooms with the door. Use this INSTEAD
		of moveInto!
	
	new verbs: go


*********/

class Door extends Lockable {

	var $isLockable;

	var $autoOpen;

	var $destoids;

	function linkTitle() { return $this->desc; }
	function Desc() {
		$a = $this->desc . " " . ucfirst($this->theName()) . " is " . ($this->isopen ? "open. " : ($this->isLocked ? "closed and locked. " : "closed. "));

		return $a;
	}
	function listContents() { return ""; }
	function Door() {
		Lockable::Lockable();
		$this->isLockable = FALSE;
		$this->isLocked = FALSE;
		$this->isopen = FALSE;
		$this->autoOpen = TRUE;
		$this->name = "door";
		$this->isFixed = TRUE;
		$this->destoids = array();
		array_push($this->doVerbs, "go");
	}
	function connectRooms(&$room1, &$room2) {
		// do the hack
		$room1->contains[$this->oid] = $this->oid;
		$room2->contains[$this->oid] = $this->oid;
		$this->destoids[$room1->oid] = $room2->oid;
		$this->destoids[$room2->oid] = $room1->oid;
	}
	function doVerbVerify($verb) {

		global $_allobjs;
		switch ($verb) {
		case "go":
			$f = $_allobjs['_Me']->location;

			if (array_key_exists($f, $this->destoids)) {
				return TRUE;
			} else {
				return FALSE;
			}
		case "unlock":
		case "lock":
		case "unlockwith":
		case "lockwith":
			if ($this->isLockable) return parent::doVerbVerify($verb); else return FALSE;

		default:
			return parent::doVerbVerify($verb);
		}
	}
	
	function verbDo($verb) {
		if ($verb == "go") return "Go through " . $this->theName(); else return parent::verbDo($verb);
	}
	function verbIng($verb) {
		if ($verb == "go") return "Going through " . $this->theName(); else return parent::verbIng($verb);
	}

	function doVerbHandle($verb) {
		global $_allobjs;
		// handle go
		$f = $_allobjs['_Me']->location;

		if ($verb == "go") {
			if ($this->isopen == FALSE) {
				if ($this->autoOpen) {
					if ($this->isLocked) {
						if ($this->keylessUnlock) {
							$this->isLocked = FALSE;
							print "<div class='descpane'>Unlocking and opening the door...</div>";
							$_allobjs['_pause'] = TRUE;
						} else {
							print "<div class='descpane'>The door is locked.</div>";
							return;
						}
					
					} else {
						$this->isopen = TRUE;
						print "<div class='descpane'>Opening the door...</div>";
						$_allobjs['_pause'] = TRUE;
					}

				} else {
					print "<div class='descpane'>The door is closed.</div>";
					return;
				}
			}

			$a = $_allobjs[$this->destoids[$f]];
			return $a->doVerbHandle("go");


		} else return parent::doVerbHandle($verb);

	}

	function ioVerbVerify($verb, $dobj) { // eliminate container functionality :)
		if ($verb == "putin") return FALSE;
		return parent::ioVerbVerify($verb, $dobj);
	}


}

/**************
	MessageFuse: A daemon which automatically which selects a message from
		a given set either at random or sequentially and displays it, one messages
		per turn.

	New/Updated Properties:
		$Messages : an array containing the messages to display.  Use blank strings for no message.
		$Loop : indicate how the messages should be played. TRUE means the messages are played
			sequentially and looped.  FALSE means the messages are played sequentially,
			but the fuse disables once they have all been played.  NULL means the messages
			are played randomly.

************/

class MessageFuse extends Fuse {
	var $Messages;
	var $Loop;

	var $currmsg;

	
	function MessageFuse() {
		Fuse::Fuse();
		$this->Messages = array();
		$this->Loop = FALSE;
		$this->currmsg = 0;
	}

	function fuseProcess($verb, $itemoid, $dobjoid) {
		$this->fuseleft = 1;
		if ($verb == "go") return;
		if (is_null($this->Loop)) {
			// choose a random message
			return $this->Messages[array_rand($this->Messages)];
		} else {
			// play messages in sequence
			$a = array_values($this->Messages);
			$a = $a[$this->currmsg];
			$this->currmsg = $this->currmsg + 1;
			if ($this->currmsg >= count($this->Messages)) {
				$this->currmsg = 0;
				if ($this->Loop == FALSE) $this->fuseleft = 0;
			}
			return $a;
		}
	}


}





/**** end of ad_classes.php ****/
?>
