<?php
/**** The Golden Skull, a PHPAdventure! sample game based on the TADS game
	of the same name. ****/


// This file is actually called twice.
// Once to get the game name for the session AND
// get custom classes (which are needed on each run) (stage 1)
// The second time to instantiate all the objects.  (only done once per session!) (stage 2)

if (!isset($_stage)) {
	// set the filename of your game

	session_name("goldenskull");
	$_mygamefile = "game_goldenskull.php";
	$_mygamecode = "38fh90f84qhf9f";
	include("ad_main.php");

} elseif ($_stage == 1) {
	// Declare your custom classes for this game here!
	class TrappedPedestal extends Surface {
		// we want the skull to cause a trap to go off when the user picks up something
		// off the pedestal and there is nothing else there

		function contentsChanged($entered, &$obj) { // contents changed is called whenever anything is put on or taken off the pedestal

			global $_allobjs; // this statement is needed to be able to access objects other than the current one.
					// In particular, we need access to special object _Game.


			// What we need to ask in this case is are there specific items thaty work on the pedestal, or
			// is it just the presence of any item?

			// We'll go with the presence of any item - more than one item is ok too (otherwise the puzzle would be unsolvable!)

			// $this refers to the current object.  contains is the array which lists what is inside the object.  We're only concerned with
			// how many things are here, not what they are in particular
			if (count($this->contains) == 0) {
				// the trap goes off!
				// note that lifting the skull may set it off, but actually ANY object could be the last straw.
				// because of this, we don't want to refer to an object by name in the text, but instead, call the appropriate method
				// so that the message is dynamic.
				print "<div class='descpane'>As you lift " . $obj->theName() . ", a volley of poisonous arrows is shot from the walls! You try to dodge the arrows, but they take you by surprise!</div>";
				// when overloading a verb or doing any direct output, it is necessary to handle all of the formatting
				// yourself.  In this case, we are creating a description pane to show the results of the action.

				// The _Game object has a member function called death that ends the game.
				// This format: $_allobjs['objectname'] can be used to access the properties
				// or member functions of any object.
				$_allobjs['_Game']->death();


			} else parent::contentsChanged($entered, $obj); // this is VERY important!
									// In almost all your overloaded classes, you will want the default
									// behavior to take over in some cases.  When you want the
									// default behavior, use this format:
									// parent::funcname(params ... )

		}

	}

	class ArchRoom extends Room {
		// for the arch, we want to repel people if they aren't carrying the skull
		// otherwise, they win the game
		function enterRoom(&$origoid) {
			// we want to "bounce" the user back if they aren't carrying the skull
			// otherwise, win the game

			global $_allobjs; // this statement is needed to be able to access objects other than the current one.
				// In particular, we need access to special object _Game.

			if ($_allobjs['goldskull']->isIn($_allobjs['_Me'])) { // we can use the location property to find out where
											// something is.  You should NEVER set this
											// property, always use moveInto instead.
											// isIn includes containers too
											// Don't use withMe because
											// that includes the room as well

				// in this case, win the game
				// Display a message
				print "<div class='descpane'>The archway accepts the golden skull in exchange for your passage to freedom.  Where will you go next?  What adventures will you have?  Only <b>you</b> can decide!</div>";

				// The _Game object has a member function called win that ends the game.
				// This format: $_allobjs['objectname'] can be used to access the properties
				// or member functions of any object.
				$_allobjs['_Game']->win();

			} else {
				// they don't have it!

				$origoid = NULL; // this disables transit

				// returned text is auto-formatted
				return "As your approach the glowing archway, a high-pitched squeal shocks your ears.  A voice booms, \"Only the holder of the golden skull may pass!\"";


			}


		}

	}

	class SkullFuse extends Fuse {
		// a fuse is a way to cause events to occur based on turn count

		var $haswarned; // we'll set this to true once the warning has been shown

		function SkullFuse() { // overloaded constructor called when class is first made
			$this->haswarned = FALSE;
			Fuse::Fuse(); // whenever a constructor is overloaded, you MUST call the parent constructor!
		}

		function fuseProcess() {
			global $_allobjs;
			// In this case, the fuse will cause the skull to explode and kill the user in a couple of turns

			// note that if the skull is not in the same room as the player, the explosion doesn't affect
			// them and they don't know about it.

			if ($this->haswarned == FALSE) {
				$this->haswarned = TRUE;
				if ($_allobjs['goldskull']->withMe()) // this tells us if the skull is with the player or not!
					$r = "The skull begins to sputter and spark!  It could explode at any moment!";
				$this->fuseleft = 5; // give them two more turns
			} else {
				// curtains
				if ($_allobjs['goldskull']->withMe()) {// this tells us if the skull is with the player or not!

					SendHeader(); // because fuse processing is pre-initialization, this must be called!
					print "<div class='descpane'>The skull explodes in a violent fury!</div>";
					$_allobjs['_Game']->death();
				} else {
					$r = "You hear a muffled explosion in the distance.";
					$_allobjs['goldskull']->moveOut(); // this makes it go away
					// note that at this point the game is unwinnable.  You may wish to simply end it
					// because of this.
				}
			}

			return $r;
		}

	}
	
	class GoldenSkull extends Carryable {
		// we'll use a custom take method to give a scary message and set the fuse
		// only if they haven't seen it before.
		
		function doVerbHandle($verb) {
			global $_allobjs;
			if ($verb == "take") { // if they take it
				// post the warning
				print "<div class='descpane'>As you pick up the skull, you notice that it immediately becomes hot to the touch.  You think you hear something burning inside!</div>";

				// engage the fuse!
				$_allobjs['skullfuse']->fuseleft = 5; // two turns until the warning
				return parent::doVerbHandle($verb); // allow normal behavior (i.e. they can still pick it up)

			} else parent::doVerbHandle($verb); // anything else, default behavior

		}

	}
	


} else {
	// Note: All object definitions must be at top-level (i.e. here, not in functions) or they will be lost!

	// Default game, startroom and me objects are already declared.

	$_Game->name = "The Golden Skull";
	$_Game->version = "1.1";
	$_Game->desc = "A sample PHPAdventure! game based on the TADS game of the same name.";
	$_Game->footermsg = "The Golden Skull translated from TADS by Steven Kollmansberger";
	$_Game->initmsg = "This game is a short sample to demonstrate how to program in PHPAdventure!";

	$_Startroom->name = "Outside Cave";
	// Notice the use of the addGo member function:
	// This function allows you to create a link from one room to another.  It generates the link and also updates
	// the room's "cango" list.  The first parameter is the name of the destination room, the second is the text
	// to appear inside the link.  We use the name of the destination instead of the destination itself so
	// you can link to rooms not yet defined.
	$_Startroom->desc = "You're standing in the bright sunlight just outside of  " . $_Startroom->addGo("cave", "a large, dark, forboding cave"). ". Nearby, a " . $_Startroom->addGo("arch", "glowing archway") . " stands between you and the rest of the universe.";


	// When creating a new object of any kind, you must use the following format:
	/**
		$objectname =& new Object();
		$objectname->oid = "objectname";
		$_allobjs['objectname'] =& $objectname;
	**/ // replace all instances of objectname with the name of the object
	$cave =& new Room(); // Room is the type where the player can go there
	$cave->oid = "cave"; // notice the name cave matchs the name given in the addGo statement for _Startroom
	$_allobjs['cave'] =& $cave;

	// Ok, now we can assign properties to this room
	$cave->name = "Cave";
	$cave->desc = "You're inside a dark and musty cave.  Sunlight pours in from " . $cave->addGo("_Startroom", "the opening") . ". ";

	// The pedestal is a surface (you can set things on it) and it's also immoble.
	// By defaults, surfaces and other containers are carriable, so we'll need
	// to turn that off.  otherwise someone could just walk away with the pedestal!
	$pedestal =& new TrappedPedestal(); // custom class to perform special actions
	$pedestal->oid = "pedestal";
	$_allobjs['pedestal'] =& $pedestal; // the usual definitions
	
	// First, to turn off the carryable flag.  isFixed is the flag that can disable carry options
	$pedestal->isFixed = TRUE;
	// When we set the isFixed flag to turn,  the pedestal will no longer be listed after the room
	// description.  You can use fixedObjRef to list these items in the room description, however,
	// the object must be declared before the room is created.  In this case, we will simply let
	// the system list the object for us.
	$pedestal->isListed = TRUE;
	
	// Now we can give it a name and description
	$pedestal->name = "pedestal";
	$pedestal->desc = "The stone pedestal is mounted in the center of the cave. ";
	
	// To assign an object to a room or container, use the moveInto member function
	$pedestal->moveInto($cave);
	
	
	// We want to put something on the pedestal - the prize for this game
	// A simple object that the player can carry around is called a Carryable
	// $goldskull =& new Carryable();
	$goldskull =& new GoldenSkull();
	$goldskull->oid = "goldskull";
	$_allobjs['goldskull'] =& $goldskull;

	$goldskull->name = "golden skull";
	$goldskull->desc = "A shiny skull that appears to be made of solid gold! ";

	// We'll put the skull on the pedestal.  The system will automatically
	// list it since we did not set the isFixed flag.
	$goldskull->moveInto($pedestal);

	
	// We need an object to counterbalance the skull
	$smallrock =& new Carryable();
	$smallrock->oid = "smallrock";
	$_allobjs['smallrock'] =& $smallrock;
	
	$smallrock->name = "small rock";
	$smallrock->desc = "A small, yet relatively heavy rock. ";
	$smallrock->moveInto($cave);

	// We need a way to win the game
	// How about an arch you can walk thourhg
	$arch =& new ArchRoom(); // even though you'll never be in this room, we make it a room for movement purposes
	$arch->oid = "arch";
	$_allobjs['arch'] =& $arch;
	$arch->name = "glowing archway";


	// create the fuse.  This is what will be activated when the user picks up the skull
	$skullfuse =& new SkullFuse();
	$skullfuse->oid = "skullfuse";
	$_allobjs['skullfuse'] =& $skullfuse;
	$skullfuse->fuseleft = 0; // disabled by default

	// create an npc to talk about the skull
	$janedoe =& new NPC();
	$janedoe->oid = "janedoe";
	$_allobjs['janedoe'] =& $janedoe;
	$janedoe->name = "Jane Doe";
	$janedoe->desc = "A perfectly ordinary looking woman sits here looking perfectly disturbed.";
	$janedoe->dontKnow = "\"Do I look like an encyclopedia?\" she asks rhetorically."; // says this when asked about something unknown
	$janedoe->randomMessages[0] = "Jane sighs and checks her watch."; // things that will happen
	$janedoe->randomMessages[1] = "Jane scowls at you."; // use blank string "" to specify nothing
	$janedoe->randomMessages[2] = "Jane twiddles her fingers.";
	$janedoe->itSelf = "herself";
	// add things that she knows a special response to in ask-about
	$janedoe->askAddObject($goldskull, "\"Ah yes,\" she begins, \"I'm supposed to tell you how exciting and great the golden skull is.  It's a skull, ok?  A chunk of bone covered in gold.  Sheesh!  Some people!\"");
	$janedoe->askAddObject($janedoe, "\"They said there was a job open for an NPC, so I'm like: 'I don't want to be a bit part like last time.'  'Oh no,' they said, 'You'll be the main NPC in the game.  Center stage.'  So here I am.  Center stage, all right.  No action roles, no intrigue, just two bit lines.  Worthless.\"  She exhales sharply and slumps down."); // '
	// you could use a conversation tree if you wanted the user to be able to pursue the conversation further.
	$janedoe->moveInto($_Startroom);

}
/**** end of file game_goldenskull.php ****/
?>
